import Koa from 'koa';
import KoaRouter from 'koa-router';
import koaStatic from 'koa-static';
import compress from 'koa-compress';
import koaBody from 'koa-body';
import webpack from 'webpack';
import pug from 'pug';
import fs from 'fs';
import path from 'path';
import {
  koaWebpackDevMiddleware,
  koaWebpackHotMiddleware,
  getAssetsFile,
} from 'webpack5-config-react';
import {
  appConfig,
  webpackConfig,
  __DEV__,
  __PROD__,
  middlewarePath,
} from './config';
import { setRouter } from './utils';

let templateSuffix: string;
let templatePath: string;

export const setHtmlTemplatePath = (tempPath?: string) => {
  if (!tempPath) tempPath = path.resolve(__dirname, './template/index.pug');

  if (!fs.existsSync(tempPath)) throw new Error(`🙅 没有找到文件 ${tempPath}`);

  templatePath = tempPath;
  templateSuffix = tempPath.split('.').pop()!;
};

setHtmlTemplatePath(appConfig.template);

const app = new Koa();
const router = new KoaRouter();
const compiler = webpack(webpackConfig);
const port = appConfig.port || 3000;

__DEV__ &&
  app
    .use(
      koaWebpackDevMiddleware(compiler, {
        publicPath: webpackConfig.output?.publicPath,
      })
    )
    .use(
      koaWebpackHotMiddleware(compiler, {
        log: false,
        path: `/__webpack_hmr`,
        heartbeat: 10 * 1000,
      })
    );

app.use(async (ctx, next) => {
  ctx['appConfig'] = appConfig;
  await next();
});
app.use(koaStatic(webpackConfig.output!.path!));

app.use(
  compress({
    threshold: 2048,
    gzip: {
      flush: require('zlib').constants.Z_SYNC_FLUSH,
    },
  })
);
/** @description 读取自定义中间件 */
if (fs.existsSync(middlewarePath)) {
  fs.readdirSync(middlewarePath).forEach(item => {
    if (/\.js$/g.test(item)) {
      try {
        const middlewareExport = require(`${middlewarePath}/${item}`);
        const middleware =
          middlewareExport && middlewareExport.default
            ? middlewareExport.default
            : middlewareExport;
        middleware && app.use(middleware);
      } catch (error) {
        // 读取不到则默认跳过
        console.log(error);
      }
    }
  });
}

setRouter(router);

app.use(router.allowedMethods());
app.use(router.routes());
app.use(
  koaBody({
    multipart: true,
  })
);

app.use(async (ctx, next) => {
  await next();
  const { status, body } = ctx;
  if (status === 404 && !body) {
    try {
      const assets = getAssetsFile();
      const inject = Object.assign(
        {
          injectData: [],
          js: [],
          css: [],
          ...appConfig,
        },
        assets
      );

      ctx.body =
        templateSuffix === 'pug'
          ? pug.renderFile(templatePath, inject)
          : fs.readFileSync(templatePath, { encoding: 'utf8' });
    } catch (error) {
      ctx.status = 500;
      ctx.body = '😭  服务器出错了 😭';
    }
  }
});

app.listen(port, () =>
  console.log(`server running in localhost:${port}`)
).timeout = 60 * 60 * 1000;

export default webpackConfig;
